#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <limits.h>
#include <errno.h>
#include <time.h>

/* WARNING: This software was intentionally implemented in
   an UNSAFE manner. USE ONLY for educational purposes!
   
   D. A. Glynos 
   First exercise material
   Software Security Course
   University of Piraeus, Department of Informatics
   (c) 2014 - 2024

   BUILD instructions:

   $ gcc -Wall -o backup exercise2024_1_source.c
   change to root (e.g. su, sudo bash etc.)
   # chown root:root backup
   # chmod +s backup

   INSTALLATION instructions:

   for normal use make sure the path /backups/access/USER exists
   (e.g. as root run 
    # mkdir -p /backups/access/john
    where john is your username)

   RUN as non-root user:

   $ ./backup access.log
   415 bytes copied 
*/

char *getuser(void) {
	char *user;

	if ((user = getenv("USER")) == NULL) {
		fprintf(stderr, "Could not deduce the username\n");
	}

	return user;
}

int open_dest_file(char *pathbuf, size_t pathbuf_sz, char *user) {
	time_t curtime;
	int err, fd;

	curtime = time(NULL);
	err = snprintf(pathbuf, pathbuf_sz, "/backups/access/%s/accesslog-%u",
		 user, (unsigned int) curtime);

	if ((err == -1) || (err >= pathbuf_sz)) {
		fprintf(stderr, "Error during path construction\n");
		return -1;
	}

	fd = open(pathbuf, O_WRONLY | O_CREAT, 0644);
	if (fd == -1) {
		perror("Error while openning file for writing");
		return -1;
	}

	return fd;	
}

int copy_file(int input, int output) {
	char buffer[100];
	int nbytes;
	int total = 0;

	while((nbytes = read(input, buffer, sizeof(buffer))) > 0) {
		write(output, buffer, nbytes);
		total += nbytes;
	}

	return total;
}

int main(int argc, char *argv[]) {
	char *user, dest_path[PATH_MAX];
	int src_fd, dest_fd, copied_bytes;

	if (argc != 2) {
		fprintf(stderr, "usage: %s /path/to/access.log\n", argv[0]);
		exit(1);
	}
	
	user = getuser();
	if (user == NULL) {
		exit(1);
	}

	src_fd = open(argv[1], O_RDONLY);
	if (src_fd == -1) {
		perror("Could not open access log for reading");
		exit(1);
	}

	dest_fd = open_dest_file(dest_path, sizeof(dest_path), user);
	if (dest_fd == -1) {
		exit(1);
	}

	copied_bytes = copy_file(src_fd, dest_fd);

	close(src_fd);
	close(dest_fd);

	fprintf(stderr, "%d bytes copied\n", copied_bytes);

	return 0;
}
